@@ -24,17 +24,25 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import h5py
-import numpy as np
-import time
 import calendar
 import datetime
-
 import logging
+import os
+import time
+
+import h5py
+import numpy as np
+
+from .correct_tsm_issue import flag_pixels as flag_tsm_pixels
+
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
+
+
 LOG = logging.getLogger(__name__)
 
-import ConfigParser
-import os
 
 try:
     CONFIG_FILE = os.environ['PYGAC_CONFIG_FILE']
@@ -64,7 +72,19 @@ SUNSATANGLES_DIR = os.environ.get('SM_SUNSATANGLES_DIR', OUTDIR)
 AVHRR_DIR = os.environ.get('SM_AVHRR_DIR', OUTDIR)
 QUAL_DIR = os.environ.get('SM_AVHRR_DIR', OUTDIR)
 MISSING_DATA = -32001
-MISSING_DATA_LATLON =  -999999
+MISSING_DATA_LATLON = -999999
+
+
+def update_start_end_line(start_line, end_line, temp_start_line, temp_end_line):
+    """Update user start/end lines after data has been sliced using temporary
+    start/end lines.
+
+    Returns:
+        Updated start_line, updated end_line
+    """
+    new_start_line = max(0, start_line - temp_start_line)
+    new_end_line = min(end_line, temp_end_line) - temp_start_line
+    return new_start_line, new_end_line
 
 
 def save_gac(satellite_name,
@@ -73,32 +93,45 @@ def save_gac(satellite_name,
              ref1, ref2, ref3,
              bt3, bt4, bt5,
              sun_zen, sat_zen, sun_azi, sat_azi, rel_azi,
-             mask, qual_flags, start_line, end_line, switch=None):
+             mask, qual_flags, start_line, end_line, tsmcorr,
+             gac_file, midnight_scanline, miss_lines, switch=None):
 
- 
+    along_track = lats.shape[0]
+    last_scan_line_number = qual_flags[-1, 0]
+
+    # Determine scanline range requested by user
     start_line = int(start_line)
     end_line = int(end_line)
+    if end_line == 0:
+        # If the user specifies 0 as the last scanline, process all scanlines
+        end_line = along_track
 
-    bt3 = np.where(np.logical_or(bt3<170.0, bt3>350.0), MISSING_DATA, bt3-273.15) 
-    bt4 = np.where(np.logical_or(bt4<170.0, bt4>350.0), MISSING_DATA, bt4-273.15) 
-    bt5 = np.where(np.logical_or(bt5<170.0, bt5>350.0), MISSING_DATA, bt5-273.15) 
+    bt3 = np.where(np.logical_or(bt3 < 170.0, bt3 > 350.0),
+                   MISSING_DATA, bt3 - 273.15)
+    bt4 = np.where(np.logical_or(bt4 < 170.0, bt4 > 350.0),
+                   MISSING_DATA, bt4 - 273.15)
+    bt5 = np.where(np.logical_or(bt5 < 170.0, bt5 > 350.0),
+                   MISSING_DATA, bt5 - 273.15)
 
-    lats = np.where(np.logical_or(lats<-90.00, lats>90.00), MISSING_DATA_LATLON, lats)
-    lons = np.where(np.logical_or(lons<-180.00, lons>180.00), MISSING_DATA_LATLON, lons)
+    lats = np.where(np.logical_or(lats < -90.00, lats > 90.00),
+                    MISSING_DATA_LATLON, lats)
+    lons = np.where(np.logical_or(lons < -180.00, lons > 180.00),
+                    MISSING_DATA_LATLON, lons)
 
     sat_azi -= 180.0
     rel_azi = abs(rel_azi)
     rel_azi = 180.0 - rel_azi
 
     for array in [bt3, bt4, bt5]:
-        array[array!=MISSING_DATA]=100*array[array!=MISSING_DATA]
+        array[array != MISSING_DATA] = 100 * array[array != MISSING_DATA]
         array[mask] = MISSING_DATA
     for array in [ref1, ref2, ref3,
                   sun_zen, sat_zen, sun_azi, sat_azi, rel_azi]:
-        array *= 100        
+        array *= 100
         array[mask] = MISSING_DATA
     for array in [lats, lons]:
-        array[array!=MISSING_DATA_LATLON]= 1000.0*array[array!=MISSING_DATA_LATLON]
+        array[array != MISSING_DATA_LATLON] = 1000.0 * \
+            array[array != MISSING_DATA_LATLON]
         array[mask] = MISSING_DATA_LATLON
 
     for ref in [ref1, ref2, ref3]:
@@ -110,89 +143,121 @@ def save_gac(satellite_name,
         ref3[switch == 2] = MISSING_DATA
         bt3[switch == 2] = MISSING_DATA
 
+    for array in [ref1, ref2, ref3, bt3, bt4, bt5]:
+        array[np.isnan(array)] = MISSING_DATA
 
-
-    no_wrong_lat = np.where(lats!=MISSING_DATA_LATLON)	
-    temp_start_line = min(no_wrong_lat[0]) 
+    # Choose new temporary start/end lines if lat/lon info is invalid
+    no_wrong_lat = np.where(lats != MISSING_DATA_LATLON)
+    temp_start_line = min(no_wrong_lat[0])
     temp_end_line = max(no_wrong_lat[0])
+    if temp_start_line > start_line:
+        LOG.info('New start_line chosen (due to invalid lat/lon info) = ' + str(temp_start_line))
+    if end_line > temp_end_line:
+        LOG.info('New end_line chosen (due to invalid lat/lon info) = ' + str(temp_end_line))
 
-
-    if temp_start_line>0 or temp_start_line>start_line:
-       LOG.info('New start_line chosen (due to invalid lat/lon info) = ' + str(temp_start_line))
-    if temp_end_line < lats.shape[0] or (end_line == 0 and temp_end_line < lats.shape[0]):
-       LOG.info('New temporary end_line chosen (due to invalid lat/lon info) = ' + str(temp_end_line))
-    if end_line>temp_end_line:
-       end_line = temp_end_line
-       LOG.info('New end_line chosen (due to invalid lat/lon info) = ' + str(temp_end_line))
-
-  
-    ref1 = ref1[temp_start_line:temp_end_line+1,:].copy()
-    ref2 = ref2[temp_start_line:temp_end_line+1,:].copy()
-    ref3 = ref3[temp_start_line:temp_end_line+1,:].copy()
-    bt3 = bt3[temp_start_line:temp_end_line+1,:].copy()
-    bt4 = bt4[temp_start_line:temp_end_line+1,:].copy()
-    bt5 = bt5[temp_start_line:temp_end_line+1,:].copy()
-    sun_zen = sun_zen[temp_start_line:temp_end_line+1,:].copy()
-    sun_azi = sun_azi[temp_start_line:temp_end_line+1,:].copy()
-    sat_zen = sat_zen[temp_start_line:temp_end_line+1,:].copy()
-    sat_azi = sat_azi[temp_start_line:temp_end_line+1,:].copy()
-    rel_azi = rel_azi[temp_start_line:temp_end_line+1,:].copy()
-    lats = lats[temp_start_line:temp_end_line+1,:].copy()
-    lons = lons[temp_start_line:temp_end_line+1,:].copy()
-    qual_flags = qual_flags[temp_start_line:temp_end_line+1,:].copy()
+    # Slice data using temporary start/end lines
+    ref1 = ref1[temp_start_line:temp_end_line + 1, :].copy()
+    ref2 = ref2[temp_start_line:temp_end_line + 1, :].copy()
+    ref3 = ref3[temp_start_line:temp_end_line + 1, :].copy()
+    bt3 = bt3[temp_start_line:temp_end_line + 1, :].copy()
+    bt4 = bt4[temp_start_line:temp_end_line + 1, :].copy()
+    bt5 = bt5[temp_start_line:temp_end_line + 1, :].copy()
+    sun_zen = sun_zen[temp_start_line:temp_end_line + 1, :].copy()
+    sun_azi = sun_azi[temp_start_line:temp_end_line + 1, :].copy()
+    sat_zen = sat_zen[temp_start_line:temp_end_line + 1, :].copy()
+    sat_azi = sat_azi[temp_start_line:temp_end_line + 1, :].copy()
+    rel_azi = rel_azi[temp_start_line:temp_end_line + 1, :].copy()
+    lats = lats[temp_start_line:temp_end_line + 1, :].copy()
+    lons = lons[temp_start_line:temp_end_line + 1, :].copy()
+    miss_lines = np.sort(np.array(
+        qual_flags[0:temp_start_line, 0].tolist() +
+        miss_lines.tolist() +
+        qual_flags[temp_end_line+1:, 0].tolist()
+    ))
+    qual_flags = qual_flags[temp_start_line:temp_end_line+1, :].copy()
     xutcs = xutcs[temp_start_line:temp_end_line+1].copy()
 
+    # Update user start/end lines to the new slice
+    start_line, end_line = update_start_end_line(
+        start_line=start_line,
+        end_line=end_line,
+        temp_start_line=temp_start_line,
+        temp_end_line=temp_end_line)
 
     # Reading time from the body of the gac file
     start = xutcs[start_line].astype(datetime.datetime)
-    if end_line == 0:
-        end = xutcs[-1].astype(datetime.datetime)
-    else:
-        end = xutcs[end_line].astype(datetime.datetime)
-
+    end = xutcs[end_line].astype(datetime.datetime)
     startdate = start.strftime("%Y%m%d")
     starttime = start.strftime("%H%M%S%f")[:-5]
     enddate = end.strftime("%Y%m%d")
     endtime = end.strftime("%H%M%S%f")[:-5]
     jday = int(start.strftime("%j"))
-   
+
     # Earth-Sun distance correction factor
     corr = 1.0 - 0.0334 * np.cos(2.0 * np.pi * (jday - 2) / 365.25)
 
-   
-    total_number_of_scan_lines = end_line - start_line + 1 
-    last_scan_line_number = qual_flags[-1,0]
+    # Slice scanline range requested by user
+    ref1 = ref1[start_line:end_line+1, :].copy()
+    ref2 = ref2[start_line:end_line+1, :].copy()
+    ref3 = ref3[start_line:end_line+1, :].copy()
+    bt3 = bt3[start_line:end_line+1, :].copy()
+    bt4 = bt4[start_line:end_line+1, :].copy()
+    bt5 = bt5[start_line:end_line+1, :].copy()
+    sun_zen = sun_zen[start_line:end_line+1, :].copy()
+    sun_azi = sun_azi[start_line:end_line+1, :].copy()
+    sat_zen = sat_zen[start_line:end_line+1, :].copy()
+    sat_azi = sat_azi[start_line:end_line+1, :].copy()
+    rel_azi = rel_azi[start_line:end_line+1, :].copy()
+    lats = lats[start_line:end_line+1, :].copy()
+    lons = lons[start_line:end_line+1, :].copy()
+    qual_flags = qual_flags[start_line:end_line+1, :].copy()
+    xutcs = xutcs[start_line:end_line+1].copy()
 
+    if midnight_scanline is not None:
+        # Update midnight scanline to the final scanline range
+        midnight_scanline -= (temp_start_line + start_line)
 
-    if end_line>0:
-       ref1 = ref1[start_line:end_line+1,:].copy()
-       ref2 = ref2[start_line:end_line+1,:].copy()
-       ref3 = ref3[start_line:end_line+1,:].copy()
-       bt3 = bt3[start_line:end_line+1,:].copy()
-       bt4 = bt4[start_line:end_line+1,:].copy()
-       bt5 = bt5[start_line:end_line+1,:].copy()
-       sun_zen = sun_zen[start_line:end_line+1,:].copy()
-       sun_azi = sun_azi[start_line:end_line+1,:].copy()
-       sat_zen = sat_zen[start_line:end_line+1,:].copy()
-       sat_azi = sat_azi[start_line:end_line+1,:].copy()
-       rel_azi = rel_azi[start_line:end_line+1,:].copy()
-       lats = lats[start_line:end_line+1,:].copy()
-       lons = lons[start_line:end_line+1,:].copy()
-       qual_flags = qual_flags[start_line:end_line+1,:].copy()
-        
+        # Set midnight scanline to None if it has been removed due to invalid
+        # lat/lon info (< 0) or lies outside the user defined scanline range
+        if midnight_scanline < 0 or (midnight_scanline > end_line or
+                                     midnight_scanline < start_line):
+            midnight_scanline = None
 
-    avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
+    # Compute total number of scanlines
+    total_number_of_scan_lines = end_line - start_line + 1
+
+    # Correct for temporary scan motor issue.
+    #
+    # TODO: The thresholds in tsm.flag_pixels() were derived from the final
+    #       pygac output, that's why the correction is applied here. It would
+    #       certainly be more consistent to apply the correction in GACReader,
+    #       but that requires a new threshold analysis.
+    if tsmcorr:
+        LOG.info('Correcting for temporary scan motor issue')
+        tic = datetime.datetime.now()
+        (ref1, ref2, bt3, bt4, bt5, ref3) = flag_tsm_pixels(channel1=ref1,
+                                                            channel2=ref2,
+                                                            channel3b=bt3,
+                                                            channel4=bt4,
+                                                            channel5=bt5,
+                                                            channel3a=ref3,
+                                                            fillv=MISSING_DATA)
+        LOG.debug('TSM correction took: %s', str(datetime.datetime.now() - tic))
+
+    avhrrGAC_io(satellite_name, xutcs, startdate, enddate, starttime, endtime,
                 lats, lons, ref1, ref2, ref3, bt3, bt4, bt5,
-                sun_zen, sat_zen, sun_azi, sat_azi, rel_azi, qual_flags, 
-		start_line, end_line, total_number_of_scan_lines,
-                last_scan_line_number, corr)
+                sun_zen, sat_zen, sun_azi, sat_azi, rel_azi, qual_flags,
+                start_line, end_line, total_number_of_scan_lines,
+                last_scan_line_number, corr, gac_file, midnight_scanline,
+                miss_lines)
 
 
-def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
+def avhrrGAC_io(satellite_name, xutcs, startdate, enddate, starttime, endtime,
                 arrLat_full, arrLon_full, ref1, ref2, ref3, bt3, bt4, bt5,
-                arrSZA, arrSTZ, arrSAA, arrSTA, arrRAA, qual_flags, 
-		start_line, end_line, total_number_of_scan_lines,
-                last_scan_line_number, corr):
+                arrSZA, arrSTZ, arrSAA, arrSTA, arrRAA, qual_flags,
+                start_line, end_line, total_number_of_scan_lines,
+                last_scan_line_number, corr, gac_file, midnight_scanline,
+                miss_lines):
     import os
 
     # Calculate start and end time in sec1970
@@ -207,7 +272,7 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
                                    satellite_name + '_99999_' +
                                    startdate + 'T' + starttime + 'Z_' +
                                    enddate + 'T' + endtime + 'Z.h5'))
-    
+
     LOG.info('Filename: ' + str(os.path.basename(ofn)))
 
     fout = h5py.File(ofn, "w")
@@ -222,17 +287,17 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
                                 data=arrLat_full)
     dset8 = fout.create_dataset("/where/lon/data", dtype='int32',
                                 data=arrLon_full)
-
+    del dset8
     channellist = []
-    channellist.append("channel1")
-    channellist.append("channel2")
-    channellist.append("channel3b")
-    channellist.append("channel4")
-    channellist.append("channel5")
-    channellist.append("channel3a")
+    channellist.append("channel1".encode('utf8'))
+    channellist.append("channel2".encode('utf8'))
+    channellist.append("channel3b".encode('utf8'))
+    channellist.append("channel4".encode('utf8'))
+    channellist.append("channel5".encode('utf8'))
+    channellist.append("channel3a".encode('utf8'))
     dset10 = fout.create_dataset("/how/channel_list",
                                  data=channellist)
-
+    del dset10
     # Attributes directly on highest level groups
     g1 = fout.require_group("/image1")
     g2 = fout.require_group("/image2")
@@ -389,7 +454,7 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
 
     # SHq: Is the sun_earth_distance correction applied?
     g1.attrs["sun_earth_distance_correction_applied"] = np.string_("TRUE")
-    g1.attrs["sun_earth_distance_correction_factor"] = corr 
+    g1.attrs["sun_earth_distance_correction_factor"] = corr
     g2.attrs["sun_earth_distance_correction_applied"] = np.string_("TRUE")
     g2.attrs["sun_earth_distance_correction_factor"] = corr
     # No attributes on 'how' for image3,4,5
@@ -405,6 +470,7 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
     g10.attrs["platform"] = np.string_(satellite_name)
     g10.attrs["instrument"] = np.string_("avhrr")
     g10.attrs["orbit_number"] = np.int32(99999)
+    g10.attrs["gac_file"] = np.string_(gac_file)
     g10.attrs["software"] = np.string_("pyGAC")
     g10.attrs["version"] = np.string_("1.0")
 
@@ -416,7 +482,7 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
                        (OUTPUT_FILE_PREFIX + '_sunsatangles_' +
                         satellite_name + '_99999_' + startdate +
                         'T' + starttime + 'Z_' +
-                        enddate + 'T' + endtime + 'Z.h5')) 
+                        enddate + 'T' + endtime + 'Z.h5'))
 
     LOG.info('Filename: ' + str(os.path.basename(ofn)))
     fout = h5py.File(ofn, "w")
@@ -431,6 +497,8 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
     dset7 = fout.create_dataset("/where/lon/data", dtype='int32',
                                 data=arrLon_full)
 
+    del dset4, dset5, dset6, dset7
+
     # Attributes directly on highest level groups
     g1 = fout.require_group("/image1")
     g2 = fout.require_group("/image2")
@@ -491,7 +559,7 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
 
     g3.attrs["product"] = np.string_("SSAZD")
     g3.attrs["quantity"] = np.string_("DEG")
-    g3.attrs["dataset_name"] = np.string_( \
+    g3.attrs["dataset_name"] = np.string_(
         'Relative satellite-sun azimuth angle')
     g3.attrs["units"] = np.string_('Deg')
     g3.attrs["gain"] = np.float32(0.01)
@@ -566,26 +634,26 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
     g9.attrs["platform"] = np.string_(satellite_name)
     g9.attrs["instrument"] = np.string_("avhrr")
     g9.attrs["orbit_number"] = np.int32(99999)
+    g9.attrs["gac_file"] = np.string_(gac_file)
     g9.attrs["software"] = np.string_("pyGAC")
     g9.attrs["version"] = np.string_("1.0")
 
     fout.close()
 
-
     LOG.info('Quality flags will be ' +
              'written to ' + str(QUAL_DIR))
     ofn = os.path.join(QUAL_DIR,
                        (OUTPUT_FILE_PREFIX + '_qualflags_' +
                         satellite_name + '_99999_' + startdate +
                         'T' + starttime + 'Z_' +
-                        enddate + 'T' + endtime + 'Z.h5')) 
+                        enddate + 'T' + endtime + 'Z.h5'))
 
     LOG.info('Filename: ' + str(os.path.basename(ofn)))
     fout = h5py.File(ofn, "w")
 
-    dset1 = fout.create_dataset("/qual_flags/data", dtype='int16', data=qual_flags)
-
     g1 = fout.require_group("/qual_flags")
+    dset1 = g1.create_dataset("data", dtype='int16', data=qual_flags)
+    del dset1
 
     g1.attrs["product"] = np.string_("QFLAG")
     g1.attrs["quantity"] = np.string_("INT")
@@ -599,8 +667,18 @@ def avhrrGAC_io(satellite_name, startdate, enddate, starttime, endtime,
     g1.attrs["endtime"] = np.string_(endtime[0:6])
     g1.attrs["startdate"] = np.string_(startdate)
     g1.attrs["enddate"] = np.string_(enddate)
+    g1.attrs["gac_file"] = np.string_(gac_file)
     g1.attrs["total_number_of_data_records"] = total_number_of_scan_lines
-    g1.attrs["last_scan_line_number"] = last_scan_line_number 
+    g1.attrs["last_scan_line_number"] = last_scan_line_number
+
+    g2 = fout.require_group("/ancillary")
+    dset2 = g2.create_dataset("missing_scanlines", dtype='int16',
+                              data=miss_lines)
+    del dset2
+    dset3 = g2.create_dataset("scanline_timestamps", dtype='int64',
+                              data=xutcs.astype('int64'))
+    dset3.attrs['units'] = 'Milliseconds since 1970-01-01 00:00:00 UTC'
+    dset3.attrs['calendar'] = 'standard'
+    g2.attrs["midnight_scanline"] = np.string_(midnight_scanline)
 
     fout.close()
-
