@@ -35,15 +35,16 @@ http://www.ncdc.noaa.gov/oa/pod-guide/ncdc/docs/podug/html/c3/sec3-1.htm
 
 """
 
-import datetime
-import logging
+from __future__ import print_function
 
 import numpy as np
-
+from pygac.gac_reader import GACReader, inherit_doc
 import pygac.geotiepoints as gtp
+from .correct_tsm_issue import TSM_AFFECTED_INTERVALS_POD
+import datetime
 from pygac import gac_io
-from pygac.gac_reader import GACReader
 
+import logging
 LOG = logging.getLogger(__name__)
 
 # common header
@@ -53,7 +54,6 @@ header0 = np.dtype([("noaa_spacecraft_identification_code", ">u1"),
                     ("number_of_scans", ">u2"),
                     ("end_time", ">u2", (3, ))])
 
-
 # until 8 september 1992
 header1 = np.dtype([("noaa_spacecraft_identification_code", ">u1"),
                     ("data_type_code", ">u1"),
@@ -146,7 +146,7 @@ header3 = np.dtype([("noaa_spacecraft_identification_code", ">u1"),
 
 scanline = np.dtype([("scan_line_number", ">i2"),
                      ("time_code", ">u2", (3, )),
-                     #("time_code", ">u1", (6, )),
+                     # ("time_code", ">u1", (6, )),
                      ("quality_indicators", ">u4"),
                      ("calibration_coefficients", ">i4", (10, )),
                      ("number_of_meaningful_zenith_angles_and_earth_location_appended",
@@ -160,9 +160,11 @@ scanline = np.dtype([("scan_line_number", ">i2"),
                      ("spare3", "u2", (11, ))])
 
 
-class PODReader(GACReader):
+@inherit_doc
+class GACPODReader(GACReader):
 
-    spacecrafts_orbital = {2: 'noaa 6',
+    spacecrafts_orbital = {25: 'tiros n',
+                           2: 'noaa 6',
                            4: 'noaa 7',
                            6: 'noaa 8',
                            7: 'noaa 9',
@@ -171,7 +173,8 @@ class PODReader(GACReader):
                            5: 'noaa 12',
                            3: 'noaa 14',
                            }
-    spacecraft_names = {2: 'noaa6',
+    spacecraft_names = {25: 'tirosn',
+                        2: 'noaa6',
                         4: 'noaa7',
                         6: 'noaa8',
                         7: 'noaa9',
@@ -181,16 +184,32 @@ class PODReader(GACReader):
                         3: 'noaa14',
                         }
 
+    tsm_affected_intervals = TSM_AFFECTED_INTERVALS_POD
+
+    def correct_scan_line_numbers(self, plot=False):
+        # Perform common corrections first.
+        super(GACPODReader, self).correct_scan_line_numbers(plot=plot)
+
+        # cleaning up the data
+        min_scanline_number = np.amin(np.absolute(self.scans["scan_line_number"][:]))
+        if self.scans["scan_line_number"][0] == self.scans["scan_line_number"][-1] + 1:
+            while self.scans["scan_line_number"][0] != min_scanline_number:
+                self.scans = np.roll(self.scans, -1)
+        else:
+            while self.scans["scan_line_number"][0] != min_scanline_number:
+                self.scans = self.scans[1:]
+
+        self.scans = self.scans[self.scans["scan_line_number"] != 0]
+
     def read(self, filename):
+        super(GACPODReader, self).read(filename=filename)
+
         # choose the right header depending on the date
         with open(filename) as fd_:
             head = np.fromfile(fd_, dtype=header0, count=1)[0]
-            year = head["start_time"][0] >> 9
-            year = np.where(year > 75, year + 1900, year + 2000)
-            jday = (head["start_time"][0] & 0x1FF)
+            year, jday, _ = self.decode_timestamps(head["start_time"])
 
-            start_date = (datetime.date(year, 1, 1) +
-                          datetime.timedelta(days=int(jday) - 1))
+            start_date = (datetime.date(year, 1, 1) + datetime.timedelta(days=int(jday) - 1))
 
             if start_date < datetime.date(1992, 9, 8):
                 header = header1
@@ -201,95 +220,56 @@ class PODReader(GACReader):
 
         with open(filename) as fd_:
             self.head = np.fromfile(fd_, dtype=header, count=1)[0]
-            scans = np.fromfile(fd_,
-                                dtype=scanline,
-                                count=self.head["number_of_scans"])
-
-        # cleaning up the data
-        min_scanline_number = np.amin(
-            np.absolute(scans["scan_line_number"][:]))
-        if scans["scan_line_number"][0] == scans["scan_line_number"][-1] + 1:
-            while scans["scan_line_number"][0] != min_scanline_number:
-                scans = np.roll(scans, -1)
-        else:
-            while scans["scan_line_number"][0] != min_scanline_number:
-                scans = scans[1:]
-
-        self.scans = scans[scans["scan_line_number"] != 0]
-
+            self.scans = np.fromfile(fd_,
+                                     dtype=scanline,
+                                     count=self.head["number_of_scans"])
+        self.correct_scan_line_numbers()
         self.spacecraft_id = self.head["noaa_spacecraft_identification_code"]
+        if self.spacecraft_id == 1 and start_date < datetime.date(1982, 1, 1):
+            self.spacecraft_id = 25
         self.spacecraft_name = self.spacecraft_names[self.spacecraft_id]
         LOG.info(
             "Reading %s data", self.spacecrafts_orbital[self.spacecraft_id])
 
         return self.head, self.scans
 
-    def get_times(self):
-        if self.utcs is None:
-            year = self.scans["time_code"][:, 0] >> 9
-            year = np.where(year > 75, year + 1900, year + 2000)
-            jday = (self.scans["time_code"][:, 0] & 0x1FF)
-            msec = ((np.uint32(self.scans["time_code"][:, 1] & 2047) << 16) |
-                    (np.uint32(self.scans["time_code"][:, 2])))
+    def get_header_timestamp(self):
+        year, jday, msec = self.decode_timestamps(self.head["start_time"])
+        try:
+            return self.to_datetime(self.to_datetime64(year=year, jday=jday,
+                                                       msec=msec))
+        except ValueError as err:
+            raise ValueError('Corrupt header timestamp: {0}'.format(err))
 
-            # print jday[10150:10171]
-            # print msec[10150:10171]
-            jday = np.where(np.logical_or(jday < 1, jday > 366),
-                            np.median(jday), jday)
-            if_wrong_jday = np.ediff1d(jday, to_begin=0)
-            jday = np.where(if_wrong_jday < 0, max(jday), jday)
+    @staticmethod
+    def decode_timestamps(encoded):
+        """
+        Decode timestamps.
+        @return: year, day of year, milliseconds since 00:00
+        """
+        ndims = len(encoded.shape)
+        if ndims == 1:
+            # Single header timestamp
+            enc0 = encoded[0]
+            enc1 = encoded[1]
+            enc2 = encoded[2]
+        elif ndims == 2:
+            # Scanline timestamps
+            enc0 = encoded[:, 0]
+            enc1 = encoded[:, 1]
+            enc2 = encoded[:, 2]
+        else:
+            raise ValueError('Invalid timestamp dimension')
 
-            if_wrong_msec = np.where(msec < 1)
-            if_wrong_msec = if_wrong_msec[0]
-            if len(if_wrong_msec) > 0:
-                if if_wrong_msec[0] != 0:
-                    msec = msec[0] + 0.5 * 1000.0 * \
-                        (self.scans["scan_line_number"] - 1)
-                else:
-                    msec = np.median(msec - 0.5 * 1000.0 *
-                                     (self.scans["scan_line_number"] - 1))
+        year = enc0 >> 9
+        year = np.where(year > 75, year + 1900, year + 2000)
+        jday = (enc0 & 0x1FF)
+        msec = ((np.uint32(enc1 & 2047) << 16) | (np.uint32(enc2)))
 
-            if_wrong_msec = np.ediff1d(msec, to_begin=0)
-            #msec = np.where(np.logical_or(if_wrong_msec<-1000, if_wrong_msec>1000), msec[0] + 0.5 * 1000.0 * (self.scans["scan_line_number"] - 1), msec)
-            msec = np.where(np.logical_and(np.logical_or(if_wrong_msec < -1000, if_wrong_msec > 1000),
-                                           if_wrong_jday != 1), msec[0] + 0.5 * 1000.0 * (self.scans["scan_line_number"] - 1), msec)
+        return year, jday, msec
 
-            self.utcs = (((year - 1970).astype('datetime64[Y]')
-                          + (jday - 1).astype('timedelta64[D]')).astype('datetime64[ms]')
-                         + msec.astype('timedelta64[ms]'))
-            self.times = self.utcs.astype(datetime.datetime)
-
-            # print if_wrong_jday[10150:10171];
-            # print if_wrong_msec[10150:10171];
-            # print jday[10150:10171]
-            # print msec[10150:10171]
-            # print self.utcs[10150:10171].astype(datetime.datetime)
-            # checking if year value is out of valid range
-            if_wrong_year = np.where(np.logical_or(year < 1978, year > 2015))
-            if_wrong_year = if_wrong_year[0]
-            if len(if_wrong_year) > 0:
-                # if the first scanline has valid time stamp
-                if if_wrong_year[0] != 0:
-                    year = year[0]
-                    jday = jday[0]
-                    msec = msec[0] + 0.5 * 1000.0 * \
-                        (self.scans["scan_line_number"] - 1)
-                    self.utcs = (((year - 1970).astype('datetime64[Y]')
-                                  + (jday - 1).astype('timedelta64[D]')).astype('datetime64[ms]')
-                                 + msec.astype('timedelta64[ms]'))
-                    self.times = self.utcs.astype(datetime.datetime)
-                # Otherwise use median time stamp
-                else:
-                    year = np.median(year)
-                    jday = np.median(jday)
-                    msec = np.median(msec - 0.5 * 1000.0 *
-                                     (self.scans["scan_line_number"] - 1))
-                    self.utcs = (((year - 1970).astype('datetime64[Y]')
-                                  + (jday - 1).astype('timedelta64[D]')).astype('datetime64[ms]')
-                                 + msec.astype('timedelta64[ms]'))
-                    self.times = self.utcs.astype(datetime.datetime)
-
-        return self.utcs
+    def _get_times(self):
+        return self.decode_timestamps(self.scans["time_code"])
 
     def adjust_clock_drift(self):
         """Adjust the geolocation to compensate for the clock error.
@@ -416,7 +396,7 @@ class PODReader(GACReader):
 
 def main(filename, start_line, end_line):
     tic = datetime.datetime.now()
-    reader = PODReader()
+    reader = GACPODReader()
     reader.read(filename)
     reader.get_lonlat()
     reader.adjust_clock_drift()
@@ -425,7 +405,7 @@ def main(filename, start_line, end_line):
 
     mask, qual_flags = reader.get_corrupt_mask()
     if (np.all(mask)):
-        print "ERROR: All data is masked out. Stop processing"
+        print("ERROR: All data is masked out. Stop processing")
         raise ValueError("All data is masked out.")
 
     gac_io.save_gac(reader.spacecraft_name,
@@ -437,7 +417,11 @@ def main(filename, start_line, end_line):
                     channels[:, :, 3],
                     channels[:, :, 4],
                     sun_zen, sat_zen, sun_azi, sat_azi, rel_azi,
-                    mask, qual_flags, start_line, end_line)
+                    mask, qual_flags, start_line, end_line,
+                    reader.is_tsm_affected(),
+                    reader.filename,
+                    reader.get_midnight_scanline(),
+                    reader.get_miss_lines())
     LOG.info("pygac took: %s", str(datetime.datetime.now() - tic))
 
 
